

"""test_flow_on_plane.py
Test the ability of ANUGA to maintain flow and match normal depth on an inclined plane
using constant inflow thru the upstream bdry at normal_depth and equivalent momentum
The plane is 20m wide by 400m long dipping at various slopes with a range of roughnesses 
A recording gauges is located  200m downstream
"""

verbose = True
import numpy as num

#------------------------------------------------------------------------------
# Import necessary modules
#------------------------------------------------------------------------------
from anuga.abstract_2d_finite_volumes.mesh_factory import rectangular_cross
from anuga.shallow_water import Domain
from anuga.shallow_water.shallow_water_domain import Reflective_boundary
from anuga.shallow_water.shallow_water_domain import Dirichlet_boundary
from anuga.shallow_water.shallow_water_domain import Transmissive_Momentum_Set_Stage_boundary
from anuga.shallow_water.shallow_water_domain import Inflow
from anuga.shallow_water.data_manager import get_flow_through_cross_section
from anuga.abstract_2d_finite_volumes.util import sww2csv_gauges, csv2timeseries_graphs


#------------------------------------------------------------------------------
# Setup computational domain
#------------------------------------------------------------------------------

finaltime = 1800.0    # 1/2 hour

length   = 400.
width    = 20.
dx = dy  = 2          # Resolution: of grid on both axes
ref_flow = 20.0       # Inflow at head of plane

points, vertices, boundary = rectangular_cross(int(length/dx), int(width/dy),
                                               len1=length, len2=width)

for mannings_n in [0.150, 0.070, 0.03, 0.015]:
    # Loop over a range of roughnesses  
    
    for slope in [1.0/1000, 1.0/300, 1.0/100]:
        # Loop over a range of bedslopes  
        

        domain = Domain(points, vertices, boundary)   
        domain.set_name('test_flowhydraulics_on_plane')              # Output name


        #------------------------------------------------------------------------------
        # Setup initial conditions
        #------------------------------------------------------------------------------

        def topography(x, y):
            z=-x * slope
            return z

        domain.set_quantity('elevation', topography)  # Use function for elevation
        domain.set_quantity('friction', mannings_n)   # Constant friction of conc surface   
        domain.set_quantity('stage',
                            expression='elevation')   # Dry initial condition


        #------------------------------------------------------------------------------
        # Seup Inflow and compute flow characteristics
        #------------------------------------------------------------------------------
      
        # Compute normal depth, Froude and momentum on plane using Mannings equation
        normal_depth  =(ref_flow*mannings_n/(slope**0.5*width))**0.6
        flow_velocity = ref_flow/(width*normal_depth)
        xmomentum     = flow_velocity*normal_depth
        froude_no     = flow_velocity/(9.8*normal_depth)**0.5
        if verbose:
            print
            print 'Slope:', slope, 'Mannings n:', mannings_n, 'Velocity:',flow_velocity,'Froude:',froude_no


        #------------------------------------------------------------------------------
        # Setup boundary conditions
        #------------------------------------------------------------------------------

        Br = Reflective_boundary(domain)                       # Solid reflective wall
        Bo = Dirichlet_boundary([-(slope*length)+normal_depth, xmomentum, 0])  # Outflow stage at normal depth
        Bi = Dirichlet_boundary([normal_depth, xmomentum, 0])  # Inflow stage at normal depth
       
        domain.set_boundary({'left': Bi, 'right': Bo, 'top': Br, 'bottom': Br})

        #------------------------------------------------------------------------------
        # Evolve system through time
        #------------------------------------------------------------------------------
        
        for t in domain.evolve(yieldstep=100.0, finaltime=finaltime):
            if verbose :
                print domain.timestepping_statistics()


        #------------------------------------------------------------------------------
        # Compute depth and check final depth against normal depth
        #------------------------------------------------------------------------------

        
        # Store temporal pattern at 200m in a csv file to check for steady state free of waves
        sww2csv_gauges('test_flowhydraulics_on_plane.sww','plane_gauge_locations.csv',quantities=['stage','elevation'],verbose=True)

        # Compute domain depth in middle of plane at 200m
        x=200.0 
        y=10.00
        w = domain.get_quantity('stage').get_values(interpolation_points=[[x, y]])[0]
        z = domain.get_quantity('elevation').get_values(interpolation_points=[[x, y]])[0]
        domain_depth = w-z
        
        msg = 'Predicted depth of flow at 200m was %f, should have been %f' % (domain_depth, normal_depth)
        if verbose:
            print 'Depth at 200m: ANUGA = %f, Mannings = %f' % (domain_depth, normal_depth)
        assert num.allclose(domain_depth,normal_depth, rtol=5.0e-2), msg

        #------------------------------------------------------------------------------
        # Compute flow thru flowlines at 200m - confirm equals inflow
        #------------------------------------------------------------------------------

        # Square on flowline at 200m
        q=domain.get_flow_through_cross_section([[200.0,0.0],[200.0,20.0]])
        msg = 'Predicted square on flow was %f, should have been %f' % (q, ref_flow)
        if verbose:
            print '90 degree flowline: ANUGA = %f, Ref = %f' % (q, ref_flow)
        assert num.allclose(q, ref_flow, rtol=1.0e-2), msg         
        
        # 45 degree flowline at 200m
        q=domain.get_flow_through_cross_section([[200.0,0.0],[220.0,20.0]])
        msg = 'Predicted 45 flow was %f, should have been %f' % (q, ref_flow)
        if verbose:
            print '45 degree flowline: ANUGA = %f, Ref = %f' % (q, ref_flow)
        assert num.allclose(q, ref_flow, rtol=1.0e-2), msg         
